/*
                               Grapher-3D

            Copyright (c) 1994, 1995, 1996  Michael P. Mesnier 
                            All rights reserved.

                  Please see README for copying information. 
*/

/*
 Graph.c
 Graph.c is called once the three dimensional points have been 
 projected into two dimensions.  It takes as input the horizontal
 and vertical lines stored in 'a' and 'b.' 'Points' is the number
 of points per line (resolution), and 'm' is the multiplying constant
 (zoom).
*/

#include "globals.h"
#include <stdio.h>

void graph(double a[][max_points][2],double b[][max_points][2],double m,
           int points, double centerx, double centery,
           FILE **output, char *color)
{
  int i,j;
  double xpa1,ypa1,xpa2,ypa2;
  double xpb1,ypb1,xpb2,ypb2;

  /*
   Horizontal and vertical lines are plotted at the same time.
   Pointer arithmetic is used to access array elements.
   Lines are made up of a collection of segments.  (xpa1,ypa1),
   (xpa2,ypa2) is the segment for the horizontal lines and
   the vertical lines use (xpb1,ypb1) and (xpb2,ypb2).  Really,
   all this function is doing is plotting the points and then
   "connecting the dots."
   */

  for (i=0;i<points;i++) 
    for (j=0;j<points-1;j++)
    {
      xpa1 =  *(*(*(a+i)+j)+0)*m+centerx;
      ypa1 =  centery-*(*(*(a+i)+j)+1)*m;
      xpb1 =  *(*(*(b+i)+j)+0)*m+centerx;
      ypb1 =  centery-*(*(*(b+i)+j)+1)*m;
      xpa2 =  *(*(*(a+i)+j+1)+0)*m+centerx;
      ypa2 =  centery-*(*(*(a+i)+j+1)+1)*m;
      xpb2 =  *(*(*(b+i)+j+1)+0)*m+centerx; 
      ypb2 =  centery-*(*(*(b+i)+j+1)+1)*m;

      fprintf(*output,".canvas create line %lf %lf %lf %lf -fill %s\n", xpa1, ypa1, xpa2, ypa2, color);
      fprintf(*output,".canvas create line %lf %lf %lf %lf -fill %s\n", xpb1, ypb1, xpb2, ypb2, color);
    }
}
