/*
                               Grapher-3D

            Copyright (c) 1994, 1995, 1996  Michael P. Mesnier 
                            All rights reserved.

                  Please see README for copying information. 
*/

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <strings.h>
#include "globals.h"
#define pi 3.1415927
#include <ctype.h>
#include <malloc.h>

  int main() {

  /*Subroutines*/

  void calculate(double a[][max_points][3],
                 double b[][max_points][3],
                 double c, int d, double e, double f, double g, double h, char function[100]);
  void transform(double a[][max_points][3],
                 double b[][max_points][2],
                 double c[][max_points][3],
                 double d[][max_points][2],
                 double e,double f,double g, int h);
  void graph(double a[][max_points][2],
             double b[][max_points][2],
             double c, int x,
             double centerx, double centery,
             FILE **output, char *color);
  int widgets(FILE **output);

  /*Variable Declaration*/

  double horiz[max_points][max_points][3];   /*3-D horizontal points*/
  double vert[max_points][max_points][3];    /*3-D vertical points*/
  double t_horiz[max_points][max_points][2]; /*2-D transformed*/
  double t_vert[max_points][max_points][2];  /*2-D transformed*/

  double x_rot = 0.392;
  double y_rot = 0.196;
  double z_rot = 0.196; 
  double zoom  = 30.0;
  double exag  = 3.0;
  double centerx = 150;
  double centery = 150;
  int x, y;
  int xres, yres;
  int recalculate; int retransform;
  int points = 10.0;
  char c, menu_choice;
  double xmin = -3.14;
  double xmax =  3.14;
  double ymin = -3.14;
  double ymax =  3.14;
  char postfix[100] = "xy+s"; /*Default postfix expression*/
  int i;
  int j;
  char draw[80]  = "blue"; 
  char geom[50]; 
  char tcl[100];
  FILE *input, *output;

  /*
   Start "wish" as a child process and 
   communicate via input and output
  */

  child("wish", &input, &output);
  widgets(&output);

  calculate(horiz,vert,exag, points, xmin, xmax, ymin, ymax, postfix);
  transform(vert,t_vert, horiz,t_horiz, x_rot,y_rot,z_rot, points);

  do
  {
    graph(t_horiz, t_vert, zoom, points, centerx, centery, &output, draw);
    fflush(output);
    fgets(tcl, 80, input);
    c = tcl[0];
    fprintf(output, ".canvas delete all\n");
    recalculate = false;
    retransform = false;

    /*Event Handler*/

    switch(c)
    {
      case 't': x_rot = y_rot = z_rot = 0;
                retransform = true;
                break;
      case 'x': sscanf(&tcl[1], "%lf", &x_rot);  
                retransform = true;
                break;
      case 'y': sscanf(&tcl[1], "%lf", &y_rot);  
                retransform = true;
                break;
      case 'z': sscanf(&tcl[1], "%lf", &z_rot);  
                retransform = true;
                break;
      case 'Z': sscanf(&tcl[1], "%lf", &zoom);  
                recalculate = true;
                break;
      case 'R': sscanf(&tcl[1], "%i", &points);  
                recalculate = true;
                break;
      case 'E': sscanf(&tcl[1], "%lf", &exag);  
                recalculate = true;
                break;
      case 'D': sscanf(&tcl[1], "%lf", &xmin);  
                fgets(tcl, 80, input);
                sscanf(&tcl[0], "%lf", &xmax);
                fgets(tcl, 80, input);
                sscanf(&tcl[0], "%lf", &ymin);
                fgets(tcl, 80, input);
                sscanf(&tcl[0], "%lf", &ymax);
                recalculate = true;
                break;
      case 'f': sscanf(tcl, "%s", postfix);
                fgets(tcl, 80, input);
                sscanf(&tcl[1], "%lf", &xmin);  
                fgets(tcl, 80, input);
                sscanf(&tcl[1], "%lf", &xmax);  
                fgets(tcl, 80, input);
                sscanf(&tcl[1], "%lf", &ymin);  
                fgets(tcl, 80, input);
                sscanf(&tcl[1], "%lf", &ymax);  
                fgets(tcl, 80, input);
                sscanf(&tcl[1], "%s", draw);  
                fgets(tcl, 80, input);
                sscanf(&tcl[0], "%i", &points);  
                fgets(tcl, 80, input);
                sscanf(&tcl[0], "%lf", &exag);  
                fgets(tcl, 80, input);
                sscanf(&tcl[0], "%lf", &zoom);  
                fgets(tcl, 80, input);
                sscanf(&tcl[0], "%lf", &x_rot);  
                fgets(tcl, 80, input);
                sscanf(&tcl[0], "%lf", &y_rot);  
                fgets(tcl, 80, input);
                sscanf(&tcl[0], "%lf", &z_rot);  
                fgets(tcl, 80, input);
                sscanf(&tcl[0], "%ix%i+%i+%i", &xres, &yres, &x, &y);  
                centerx = xres/2;
                centery = yres/2;
                recalculate = true;
                break;
    }
    if (recalculate)
       calculate(horiz,vert,exag, points, xmin, xmax, ymin, ymax, postfix),
       transform(horiz,t_horiz,
                 vert,t_vert,
                 x_rot,y_rot,z_rot, points); 
    else if (retransform)
       transform(horiz,t_horiz,
                 vert,t_vert,
                 x_rot,y_rot,z_rot, points);
  }
  while (c!='q');
  return 0;
}
